import Products.CMFBibliographyAT.content.folder
import Products.CMFBibliographyAT.content.article
import Products.CMFBibliographyAT.content.book
import Products.CMFBibliographyAT.content.booklet
import Products.CMFBibliographyAT.content.conference
import Products.CMFBibliographyAT.content.inbook
import Products.CMFBibliographyAT.content.incollection
import Products.CMFBibliographyAT.content.inproceedings
import Products.CMFBibliographyAT.content.manual
import Products.CMFBibliographyAT.content.mastersthesis
import Products.CMFBibliographyAT.content.misc
import Products.CMFBibliographyAT.content.phdthesis
import Products.CMFBibliographyAT.content.preprint
import Products.CMFBibliographyAT.content.proceedings
import Products.CMFBibliographyAT.content.techreport
import Products.CMFBibliographyAT.content.unpublished
import Products.CMFBibliographyAT.content.webpublished
import Products.CMFBibliographyAT.content.pdf
